@@ -6,8 +6,14 @@ from django.db import models
 from django.db.models.signals import post_save
 from django.utils.crypto import get_random_string
 from django.utils import timezone
-from django.utils.translation import ugettext_lazy as _
+from six import python_2_unicode_compatible
+import django
+if django.VERSION < (4, 0):
+    from django.utils.translation import ugettext_lazy as _
+else:
+    from django.utils.translation import gettext_lazy as _
 
+from simple_email_confirmation import get_email_address_model
 from .exceptions import (
     EmailConfirmationExpired, EmailIsPrimary, EmailNotConfirmed,
 )
@@ -39,19 +45,22 @@ class SimpleEmailConfirmationUserMixin(object):
         if email == old_email:
             return
 
-        if email not in self.confirmed_emails and require_confirmed:
+        if email not in self.get_confirmed_emails() and require_confirmed:
             raise EmailNotConfirmed()
 
         setattr(self, self.primary_email_field_name, email)
         self.save(update_fields=[self.primary_email_field_name])
         primary_email_changed.send(
-            sender=self, old_email=old_email, new_email=email,
+            sender=self.__class__,
+            user=self,
+            old_email=old_email,
+            new_email=email,
         )
 
     @property
     def is_confirmed(self):
         "Is the User's primary email address confirmed?"
-        return self.get_primary_email() in self.confirmed_emails
+        return self.get_primary_email() in self.get_confirmed_emails()
 
     @property
     def confirmed_at(self):
@@ -100,7 +109,7 @@ class SimpleEmailConfirmationUserMixin(object):
         Attempt to confirm an email using the given key.
         Returns the email that was confirmed, or raise an exception.
         """
-        address = self.email_address_set.confirm(confirmation_key, save=save)
+        address = self.email_address_set.confirm(confirmation_key, save=save, user=self)
         return address.email
 
     def add_confirmed_email(self, email):
@@ -126,7 +135,7 @@ class SimpleEmailConfirmationUserMixin(object):
         """
         try:
             address = self.email_address_set.get(email=email)
-        except EmailAddress.DoesNotExist:
+        except get_email_address_model().DoesNotExist:
             key = self.add_unconfirmed_email(email)
         else:
             if not address.is_confirmed:
@@ -154,8 +163,11 @@ class EmailAddressManager(models.Manager):
 
     def generate_key(self):
         "Generate a new random key and return it"
-        # sticking with the django defaults
-        return get_random_string()
+        # By default, a length of keys is 12. If you want to change it, set
+        # settings.SIMPLE_EMAIL_CONFIRMATION_KEY_LENGTH to integer value (max 40).
+        return get_random_string(
+            length=min(getattr(settings, 'SIMPLE_EMAIL_CONFIRMATION_KEY_LENGTH', 12), 40)
+        )
 
     def create_confirmed(self, email, user=None):
         "Create an email address in the confirmed state"
@@ -178,7 +190,11 @@ class EmailAddressManager(models.Manager):
         key = self.generate_key()
         # let email-already-exists exception propogate through
         address = self.create(user=user, email=email, key=key)
-        unconfirmed_email_created.send(sender=user, email=email)
+        unconfirmed_email_created.send(
+            sender=user.__class__,
+            user=user,
+            email=email,
+        )
         return address
 
     def confirm(self, key, user=None, save=True):
@@ -195,16 +211,31 @@ class EmailAddressManager(models.Manager):
             address.confirmed_at = timezone.now()
             if save:
                 address.save(update_fields=['confirmed_at'])
-                email_confirmed.send(sender=address.user, email=address.email)
+                email_confirmed.send(
+                    sender=address.user.__class__,
+                    user=address.user,
+                    email=address.email
+                )
 
         return address
 
 
-class EmailAddress(models.Model):
+def get_user_primary_email(user):
+    # softly failing on using these methods on `user` to support
+    # not using the SimpleEmailConfirmationMixin in your User model
+    # https://github.com/mfogel/django-simple-email-confirmation/pull/3
+    if hasattr(user, 'get_primary_email'):
+        return user.get_primary_email()
+    return user.email
+
+
+@python_2_unicode_compatible
+class AbstractEmailAddress(models.Model):
     "An email address belonging to a User"
 
     user = models.ForeignKey(
         settings.AUTH_USER_MODEL, related_name='email_address_set',
+        on_delete=models.CASCADE,
     )
     email = models.EmailField(max_length=255)
     key = models.CharField(max_length=40, unique=True)
@@ -223,8 +254,9 @@ class EmailAddress(models.Model):
     class Meta:
         unique_together = (('user', 'email'),)
         verbose_name_plural = "email addresses"
+        abstract = True
 
-    def __unicode__(self):
+    def __str__(self):
         return '{} <{}>'.format(self.user, self.email)
 
     @property
@@ -233,7 +265,8 @@ class EmailAddress(models.Model):
 
     @property
     def is_primary(self):
-        return bool(self.user.email == self.email)
+        primary_email = get_user_primary_email(self.user)
+        return bool(primary_email == self.email)
 
     @property
     def key_expires_at(self):
@@ -251,10 +284,10 @@ class EmailAddress(models.Model):
     def reset_confirmation(self):
         """
         Re-generate the confirmation key and key expiration associated
-        with this email.  Note that the previou confirmation key will
+        with this email.  Note that the previous confirmation key will
         cease to work.
         """
-        self.key = self._default_manager.generate_key()
+        self.key = get_email_address_model()._default_manager.generate_key()
         self.set_at = timezone.now()
 
         self.confirmed_at = None
@@ -262,22 +295,22 @@ class EmailAddress(models.Model):
         return self.key
 
 
+class EmailAddress(AbstractEmailAddress):
+    class Meta(AbstractEmailAddress.Meta):
+        swappable = 'SIMPLE_EMAIL_CONFIRMATION_EMAIL_ADDRESS_MODEL'
+
+
 # by default, auto-add unconfirmed EmailAddress objects for new Users
 if getattr(settings, 'SIMPLE_EMAIL_CONFIRMATION_AUTO_ADD', True):
     def auto_add(sender, **kwargs):
-        if sender == get_user_model() and kwargs['created']:
+        if sender == get_user_model() and kwargs['created'] and not kwargs['raw']:
             user = kwargs.get('instance')
-            # softly failing on using these methods on `user` to support
-            # not using the SimpleEmailConfirmationMixin in your User model
-            # https://github.com/mfogel/django-simple-email-confirmation/pull/3
-            if hasattr(user, 'get_primary_email'):
-                email = user.get_primary_email()
-            else:
-                email = user.email
-            if hasattr(user, 'add_unconfirmed_email'):
-                user.add_unconfirmed_email(email)
-            else:
-                user.email_address_set.create_unconfirmed(email)
+            email = get_user_primary_email(user)
+            if email:
+                if hasattr(user, 'add_unconfirmed_email'):
+                    user.add_unconfirmed_email(email)
+                else:
+                    user.email_address_set.create_unconfirmed(email)
 
     # TODO: try to only connect this to the User model. We can't use
     #       get_user_model() here - results in import loop.
